@@ -5,17 +5,6 @@ import utemplate.source
 from .utils import parse_qs
 
 
-template_loader = utemplate.source.Loader(".")
-
-def render(writer, tmpl_name, args=()):
-    tmpl = template_loader.load(tmpl_name)
-    for s in tmpl(*args):
-        yield from writer.awrite(s)
-
-def render_str(tmpl_name, args=()):
-    #TODO: bloat
-    tmpl = template_loader.load(tmpl_name)
-    return ''.join(tmpl(*args))
 
 def sendfd(writer, f):
     while True:
@@ -42,10 +31,8 @@ def start_response(writer, content_type="text/html", status="200"):
 
 class HTTPRequest:
 
-    def __init__(self, method, path, headers):
-        self.method = method
-        self.path = path
-        self.headers = headers
+    def __init__(self):
+        pass
 
     def read_form_data(self):
         size = int(self.headers["Content-Length"])
@@ -56,27 +43,36 @@ class HTTPRequest:
 
 class WebApp:
 
-    def __init__(self, routes=None):
+    def __init__(self, routes=None, static="static"):
         if routes:
             self.url_map = routes
         else:
             self.url_map = []
+        if static:
+            self.url_map.append((re.compile("^/static(/.+)"),
+                lambda req, resp: (yield from sendfile(resp, static + req.url_match.group(1)))))
         self.mounts = []
         self.inited = False
+        self.template_loader = utemplate.source.Loader("templates")
 
     def _handle(self, reader, writer):
-        print("================")
-        print(reader, writer)
         request_line = yield from reader.readline()
+        # TODO: bytes vs str
+        request_line = request_line.decode()
         method, path, proto = request_line.split()
         headers = {}
+        req = HTTPRequest()
         while True:
             l = yield from reader.readline()
+            # TODO: bytes vs str
+            l = l.decode()
             if l == "\r\n":
                 break
             k, v = l.split(":", 1)
             headers[k] = v.strip()
-        print((method, path, proto), headers)
+        print("================")
+        print(req, writer)
+        print(req, (method, path, proto), headers)
 
         # Find which mounted subapp (if any) should handle this request
         app = self
@@ -98,8 +94,6 @@ class WebApp:
         if not app.inited:
             app.init()
 
-        req = HTTPRequest(method, path, headers)
-
         found = False
         for pattern, handler, *extra in app.url_map:
             if path == pattern:
@@ -112,14 +106,17 @@ class WebApp:
                     found = True
                     break
         if found:
+            req.method = method
+            req.path = path
+            req.headers = headers
             req.reader = reader
-            yield from handler(writer, req)
+            yield from handler(req, writer)
         else:
             yield from start_response(writer, status="404")
             yield from writer.awrite("404\r\n")
-        print("After response write")
+        print(req, "After response write")
         yield from writer.close()
-        print("Finished processing request")
+        print(req, "Finished processing request")
 
     def mount(self, url, app):
         "Mount a sub-app at the url of current app."
@@ -143,6 +140,16 @@ class WebApp:
         # because it's alleged bloat.
         self.url_map.append((url, func, kwargs))
 
+    def render_template(self, writer, tmpl_name, args=()):
+        tmpl = self.template_loader.load(tmpl_name)
+        for s in tmpl(*args):
+            yield from writer.awrite(s)
+
+    def render_str(self, tmpl_name, args=()):
+        #TODO: bloat
+        tmpl = self.template_loader.load(tmpl_name)
+        return ''.join(tmpl(*args))
+
     def init(self):
         """Initialize a web application. This is for overriding by subclasses.
         This is good place to connect to/initialize a database, for example."""
